/*******************************************************************************
 * Copyright (c) 2018-2020 ArSysOp
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Contributors:
 *     ArSysOp - initial API and implementation
 *******************************************************************************/
package org.pgcase.xobot.basis.emf.edit;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.eclipse.emf.common.command.BasicCommandStack;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.util.EContentAdapter;
import org.eclipse.emf.edit.domain.AdapterFactoryEditingDomain;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.emf.edit.provider.ComposedAdapterFactory;
import org.eclipse.osgi.service.environment.EnvironmentInfo;
import org.osgi.service.event.Event;
import org.osgi.service.event.EventAdmin;

public abstract class EditingDomainBasedRegistry<C> implements EditingDomainRegistry<C> {

	// @see org.eclipse.e4.core.services.events.IEventBroker.DATA
	protected static final String PROPERTY_DATA = "org.eclipse.e4.data"; //$NON-NLS-1$

	protected EnvironmentInfo environmentInfo;

	protected EventAdmin eventAdmin;

	private ComposedAdapterFactory composedAdapterFactory;

	private AdapterFactoryEditingDomain editingDomain;

	private final List<String> resourceIdentifiers = new ArrayList<>();

	private EContentAdapter contentAdapter;

	private final Map<String, C> contentIndex = new HashMap<>();

	public EditingDomainBasedRegistry() {
		BasicCommandStack commandStack = new BasicCommandStack();
		editingDomain = new AdapterFactoryEditingDomain(composedAdapterFactory, commandStack,
				new HashMap<Resource, Boolean>());
	}

	protected void bindEnvironmentInfo(EnvironmentInfo environmentInfo) {
		this.environmentInfo = environmentInfo;
	}

	protected void unbindEnvironmentInfo(EnvironmentInfo environmentInfo) {
		this.environmentInfo = null;
	}

	protected void bindEventAdmin(EventAdmin eventAdmin) {
		this.eventAdmin = eventAdmin;
	}

	protected void unbindEventAdmin(EventAdmin eventAdmin) {
		this.eventAdmin = null;
	}

	protected void bindFactoryProvider(ComposedAdapterFactoryProvider factoryProvider) {
		this.composedAdapterFactory = factoryProvider.getComposedAdapterFactory();
		editingDomain.setAdapterFactory(composedAdapterFactory);
	}

	protected void unbindFactoryProvider(ComposedAdapterFactoryProvider factoryProvider) {
		this.composedAdapterFactory = null;
		editingDomain.setAdapterFactory(composedAdapterFactory);
	}

	protected void activate(Map<String, Object> properties) {
		contentAdapter = createContentAdapter();
		editingDomain.getResourceSet().eAdapters().add(contentAdapter);
	}

	protected abstract DomainContentAdapter<C, ? extends EditingDomainRegistry<C>> createContentAdapter();

	protected void deactivate(Map<String, Object> properties) {
		editingDomain.getResourceSet().eAdapters().remove(contentAdapter);
	}

	@Override
	public ComposedAdapterFactory getComposedAdapterFactory() {
		return composedAdapterFactory;
	}

	@Override
	public EditingDomain getEditingDomain() {
		return editingDomain;
	}

	protected Map<?, ?> getLoadOptions() {
		return new HashMap<>();
	}

	protected Map<?, ?> getSaveOptions() {
		return new HashMap<>();
	}

	public void loadResource(String source) throws Exception {
		URI uri = createURI(source);
		ResourceSet resourceSet = editingDomain.getResourceSet();
		Resource resource = resourceSet.createResource(uri);
		resource.load(getLoadOptions());
	}

	public void unloadResource(String source) throws Exception {
		URI uri = createURI(source);
		ResourceSet resourceSet = editingDomain.getResourceSet();
		Resource resource = resourceSet.getResource(uri, false);
		resource.unload();
		resourceSet.getResources().remove(resource);
	}

	protected URI createURI(String source) {
		String platformResource = "platform:"; //$NON-NLS-1$
		if (source.startsWith(platformResource)) {
			return URI.createURI(source);
		}
		return URI.createFileURI(source);
	}

	@Override
	public void registerResource(String identifier) {
		resourceIdentifiers.add(identifier);
		try {
			loadResource(identifier);
		} catch (Exception e) {
			e.printStackTrace();
			Logger.getLogger(this.getClass().getName()).log(Level.FINER, e.getMessage(), e);
		}
	}

	@Override
	public void unregisterResource(String identifier) {
		resourceIdentifiers.remove(identifier);
		try {
			unloadResource(identifier);
		} catch (Exception e) {
			Logger.getLogger(this.getClass().getName()).log(Level.FINER, e.getMessage(), e);
		}
	}

	@Override
	public Iterable<String> getResourceIdentifiers() {
		return Collections.unmodifiableList(resourceIdentifiers);
	}

	protected static Event createEvent(String topic, Object data) {
		Map<String, Object> properties = new HashMap<>();
		properties.put(PROPERTY_DATA, data);
		Event event = new Event(topic, properties);
		return event;
	}
	
	@Override
	public Iterable<? extends C> getRegistryContent() {
		return contentIndex.values();
	}
	
	@Override
	public C getRegistryContent(String identifier) {
		return contentIndex.get(identifier);
	}
	
	@Override
	public void registerContent(C content) {
		String identifier = resolveIdentifier(content);
		C existing = contentIndex.put(identifier, content);
		if (existing != null) {
			// FIXME: warning
		}
		eventAdmin.postEvent(createEvent(getContentCreateTopic(), content));
		// TODO Auto-generated method stub
		
	}
	
	@Override
	public void unregisterContent(String identifier) {
		C removed = contentIndex.remove(identifier);
		if (removed != null) {
			eventAdmin.postEvent(createEvent(getContentDeleteTopic(), removed));
			// TODO Auto-generated method stub
		} else {
			// FIXME: warning
		}
	}
	
}
